@@ -2,4 +2,4 @@
 Simple Python terminal (TUI) multi-line editor
 """
 
-from .editor import Editor
+from .tui_editor import TuiEditor
