@@ -29,8 +29,6 @@ class TuiEditor:
         self.col = 0
         self.height = 10  # lines for editor only, excluding status bars
         self.show_line_numbers = False
-        self._orig_termios = None
-        self._orig_sig_win_ch = None
         self.content_prefix_escape = b"\x1b[30;106m"
         self.status_prefix_escape = b"\x1b[30;102m"
         self._content = [""]
@@ -53,15 +51,17 @@ class TuiEditor:
 
     def set_text_lines(self, lines: list[str]):
         """set editor text"""
-        self._content = lines or [""]
+        self._content = lines.copy() or [""]
+        if self.tty.initialized:
+            self.update_screen()
 
     def set_text(self, text: str):
         """set editor text"""
-        self._content = text.split("\n")
+        self.set_text_lines(text.split("\n"))
 
     def get_text_lines(self):
         """get editor text"""
-        return self._content
+        return self._content.copy()
 
     def get_text(self) -> str:
         """get editor text"""
@@ -69,8 +69,9 @@ class TuiEditor:
 
     def set_status_lines(self, lines: list[str]):
         """set status bar"""
-        self._status_content = lines or [""]
-        self.update_screen_status()
+        self._status_content = lines.copy() or [""]
+        if self.tty.initialized:
+            self.update_screen_status()
 
     def set_cursor_pos(self, line: int, col: int = 0):
         """
@@ -80,6 +81,8 @@ class TuiEditor:
             self.top_line_idx = max(line - self.actual_height // 2, 0)
         self.row = line - self.top_line_idx
         self.col = col
+        if self.tty.initialized:
+            self.update_screen()
 
     def set_cursor(self):
         """set the cursor back to the editor pos"""
@@ -109,12 +112,18 @@ class TuiEditor:
 
     def loop(self):
         """main loop, reading user inputs"""
+        assert self.tty.initialized
         while True:
             buf = os.read(self.tty.fd_in, 32)
+            if buf and buf[:1] not in b"\x1b\x7f" and buf[0] > 31:
+                try:
+                    buf = buf.decode("utf8")
+                except UnicodeDecodeError:
+                    pass
             sz = len(buf)
             i = 0
             while i < sz:
-                if buf[0] == 0x1b:
+                if isinstance(buf, bytes) and buf[0] == 0x1b:
                     key = buf
                     i = len(buf)
                 else:
@@ -136,6 +145,7 @@ class TuiEditor:
 
     def update_screen(self):
         """update the screen, i.e. editor and status bar(s)"""
+        assert self.tty.initialized
         self.tty.cursor(False)
         self.tty.goto(0, 0)
         self.tty.write(self.content_prefix_escape)
@@ -161,6 +171,7 @@ class TuiEditor:
 
         :param goto: set the cursor ot the status bar, and then back to the editor
         """
+        assert self.tty.initialized
         if goto:
             self.tty.cursor(False)
             self.tty.goto(self.actual_height, 0)
@@ -315,7 +326,7 @@ class TuiEditor:
             return False
         return True
 
-    def handle_key(self, key: Union[bytes, int]):
+    def handle_key(self, key: Union[bytes, str, int]):
         """
         Handle potential edit key and update screen.
         Assumes the cursor is at the right position.
@@ -355,17 +366,19 @@ class TuiEditor:
                 self.tty.update_occupied_space()  # actual height might have decreased
                 self.update_screen()
         elif isinstance(key, int):
-            pass
-        elif key[0] == 0x1b or len(key) > 1 or ord(key) <= 31:  # other control char
-            pass
-        else:
-            cur_line = cur_line[:self.col] + str(key, "utf-8") + cur_line[self.col:]
+            return
+        elif isinstance(key, bytes) and key[0] == 0x1b:
+            return
+        elif len(key) == 1 and ord(key) <= 31:  # other control char
+            return
+        elif isinstance(key, str):
+            cur_line = cur_line[:self.col] + key + cur_line[self.col:]
             self._content[self.cur_line_idx] = cur_line
             self.col += 1
             self.update_line()
         self.on_edit()
 
-    def on_key(self, key: Union[bytes, int]) -> Union[bool, None]:
+    def on_key(self, key: Union[bytes, str, int]) -> Union[bool, None]:
         """
         Called on input key events.
 
